extend("fiskheroes:hero_basic");
loadTextures({
    "layer1": "smp:entity_layer1",
    "layer2": "smp:entity_layer2",
    "book": "smp:entity_book",
    "axe": "smp:entity_axe",
    "lights": "smp:entity_axe_light"
});
var utils = implement("fiskheroes:external/utils");

function initEffects(renderer) {

    utils.bindCloud(renderer, "fiskheroes:telekinesis", "smp:entity");
    utils.bindCloud(renderer, "fiskheroes:teleportation", "smp:entity_teleport");

    var beam = renderer.createResource("BEAM_RENDERER", "fiskheroes:mysterio_beam");
    utils.bindBeam(renderer, "fiskheroes:charged_beam", beam, "body", 0xFF0000, [
        { "firstPerson": [0, -2400.0, -50.0], "offset": [0, -2400, -50.0], "size": [5, 5] }
    ]).setParticles(renderer.createResource("PARTICLE_EMITTER", "fiskheroes:impact_charged_beam"));

    utils.bindBeam(renderer, "fiskheroes:energy_projection", beam, "rightArm", 0xFF0000, [
        { "firstPerson": [-5.5, 4.75, -10.0], "offset": [-0.5, 9.0, 0.0], "size": [1.5, 1.5] },
        { "firstPerson": [5.5, 4.75, -10.0], "offset": [0.5, 9.0, 0.0], "size": [1.5, 1.5], "anchor": "leftArm" }
    ]).setParticles(renderer.createResource("PARTICLE_EMITTER", "fiskheroes:impact_energy_projection"));

    var model = renderer.createResource("MODEL", "smp:entity_book");
    model.bindAnimation("smp:entity_book")
        .setData((entity, data) => {
            data.load(0, entity.getInterpolatedData("fiskheroes:beam_charge"));
        })
    model.texture.set("book");
    book = renderer.createEffect("fiskheroes:model").setModel(model);
    book.anchor.set("body");
    book.setRotation(0, 180, 0);
    book.setOffset(0, -16, -16);

    axe = renderer.createEffect("fiskheroes:model");
    axe.setModel(utils.createModel(renderer, "smp:entity_axe", "axe", "lights"));
    axe.anchor.set("rightArm");
}

function initAnimations(renderer) {
    parent.initAnimations(renderer);
    utils.addFlightAnimationWithLanding(renderer, "iron_man.FLIGHT", "fiskheroes:flight/iron_man.anim.json");
    utils.addHoverAnimation(renderer, "mmcw.HOVER", "fiskheroes:flight/idle/neutral");
    utils.addAnimationEvent(renderer, "FLIGHT_DIVE", "fiskheroes:iron_man_dive");

    addAnimationWithData(renderer, "iron_man.LAND", "fiskheroes:superhero_landing", "fiskheroes:dyn/superhero_landing_timer")
        .priority = -8;

    addAnimationWithData(renderer, "entity.CHARGE", "smp:entity_charge", "fiskheroes:beam_charge")
        .priority = -8;

    addAnimationWithData(renderer, "iron_man.ROLL", "fiskheroes:flight/barrel_roll", "fiskheroes:barrel_roll_timer")
        .priority = 10;

    addAnimationWithData(renderer, "basic.ENERGY_PROJ", "fiskheroes:dual_aiming", "fiskheroes:energy_projection_timer");
}

function render(entity, renderLayer, isFirstPersonArm) {
    if (entity.getData("fiskheroes:beam_charge") > 0) {
        book.setScale(entity.getData("fiskheroes:beam_charge") < 0.5 ? entity.getInterpolatedData("fiskheroes:beam_charge") * 2 : 1);
        book.render();
        if (isFirstPersonArm) {
            book.anchor.ignoreAnchor(true);
        }
        if (!isFirstPersonArm) {
            book.anchor.ignoreAnchor(false);
        }
    }
    if (entity.getData("fiskheroes:blade")) {
        axe.render()
        axe.setOffset(-4.95, -2.15, 0);
    }

    if (!isFirstPersonArm) {
        axe.setOffset(-4.95, -2.15, 0);
    }
}