extend("fiskheroes:hero_basic");
loadTextures({
    "layer1": "smp:potato_layer1",
    "layer2": "smp:potato_layer2",
    "fab1": "smp:potato_s2_layer1",
    "fab2": "smp:potato_s2_layer2",
    "cape": "smp:potato_cape",
    "mouth": "smp:potato_mouth",
    "cooked": "smp:potato_cooked"
});

var utils = implement("fiskheroes:external/utils");
var capes = implement("fiskheroes:external/capes");

var physics;
var cape;
var mouth;
var cooked;
var burnt;

function trail() {
    return "smp:potato"
}

function init(renderer) {
    parent.init(renderer);
    renderer.setTexture((entity, renderLayer) => {
        if (entity.as("DISPLAY").getDisplayType() === "FABRICATOR_PREVIEW" || entity.as("DISPLAY").getDisplayType() === "FABRICATOR_RESULT") {
            return renderLayer == "LEGGINGS" ? "fab2" : "fab1";
        }
        return renderLayer == "LEGGINGS" ? "layer2" : "layer1";
    });
}

function initEffects(renderer) {
    utils.bindBeam(renderer, "fiskheroes:heat_vision", "fiskheroes:mysterio_beam", "head", 0xEFEAB5, [
        { "firstPerson": [2.2, 0.0, 2.0], "offset": [2.2, -4, -4.0], "size": [2.5, 0.5] },
        { "firstPerson": [-2.2, 0.0, 2.0], "offset": [-2.2, -4, -4.0], "size": [2.5, 0.5] }
    ]).setParticles(renderer.createResource("PARTICLE_EMITTER", "fiskheroes:impact_energy_projection"));

    physics = renderer.createResource("CAPE_PHYSICS", null);
    physics.maxFlare = 0.5;
    physics.flareDegree = 1.5;
    physics.flareFactor = 1.2;
    physics.flareElasticity = 6.5;
    physics.setTickHandler(entity => {
        var s = entity.getData("smp:dyn/cook_timer");
        var t = entity.getData("fiskheroes:flight_timer");
        var f = entity.as("DISPLAY").getDisplayType() === "BOOK_PREVIEW" || entity.is("DISPLAY") ? 1 : 0;
        f = 1 - f * f * f;
        physics.headingAngle = 90 + 25 * t - f * 20;
        physics.restAngle = s * 40;
        physics.restFlare = f * 0.7;
        physics.idleFlutter = 0.15 + 0.25 * f;
        physics.flutterSpeed = f * 0.3;
    });

    cape = capes.create(renderer, 24, "fiskheroes:cape_default.mesh.json");
    cape.effect.texture.set("cape");
    cape.effect.width = 12;
    
    mouth = renderer.createEffect("fiskheroes:overlay");
    mouth.texture.set("mouth");

    cooked = renderer.createEffect("fiskheroes:overlay");
    cooked.texture.set("cooked");

    utils.bindTrail(renderer, trail());

    utils.addCameraShake(renderer, 0.015, 1.25, "fiskheroes:flight_timer");
    var shake = renderer.bindProperty("fiskheroes:camera_shake").setCondition(entity => {
        var flight = entity.getData("fiskheroes:flying") && entity.isSprinting() ? entity.getInterpolatedData("fiskheroes:flight_boost_timer") : 0;
        var heat_vision = entity.getInterpolatedData("fiskheroes:heat_vision_timer");
        var cook = entity.getInterpolatedData("smp:dyn/cook_timer");
        var boost = entity.getData("fiskheroes:flying") && entity.isSprinting() ? entity.getInterpolatedData("fiskheroes:dyn/flight_super_boost_timer") : 0;
        shake.factor = Math.max(Math.sin(Math.PI * flight), heat_vision * cook, 1.5 * Math.sin(Math.PI * boost));
        return true;
    });
    shake.intensity = 0.0;
}

function initAnimations(renderer) {
    parent.initAnimations(renderer);
    utils.addFlightAnimationWithLanding(renderer, "iron_man.FLIGHT", "fiskheroes:flight/iron_man.anim.json");
    utils.addHoverAnimation(renderer, "mmcw.HOVER", "fiskheroes:flight/idle/neutral");
    utils.addAnimationEvent(renderer, "FLIGHT_DIVE", "fiskheroes:iron_man_dive");

    addAnimationWithData(renderer, "iron_man.LAND", "fiskheroes:superhero_landing", "fiskheroes:dyn/superhero_landing_timer")
        .priority = -8;

    addAnimationWithData(renderer, "iron_man.ROLL", "fiskheroes:flight/barrel_roll", "fiskheroes:barrel_roll_timer")
        .priority = 10;
}

function render(entity, renderLayer, isFirstPersonArm) {
    cooked.opacity = entity.getInterpolatedData("smp:dyn/cook_timer");
    cooked.render();

    if (!isFirstPersonArm && renderLayer == "CHESTPLATE") {
        var f = entity.getInterpolatedData("fiskheroes:flight_timer");
        cape.render({
            "wind": 1 + 0.3 * f,
            "windFactor": 1 - 0.7 * f,
            "flutter": physics.getFlutter(entity),
            "flare": physics.getFlare(entity)
        });
    }
    if (renderLayer == "HELMET" && entity.getData("fiskheroes:sonic_waves")) {
        mouth.render();
    }
}